// Convolve a sound with an impulse reponse
// Music: Erik Satie - Gymnopédie No 1
// Copyright (C) 2015-2017 - Paul Bignier

version = getversion("scilab");
if version(1,1) <= 5 then
    stacksize("max");
end

p = funcprot();
funcprot(0);

// Defining the GUI
fig = figure(...
    "figure_name", "Sound convolution (convol function)",...
    "figure_size", [600 500],...
    "layout", "gridbag",...
    "background", 8,...
    "menubar_visible", "off",...
    "toolbar_visible", "off",...
    "default_axes", "off",...
    "visible", "off",...
    "tag", "main_figure");

// Frame for the uicontrols
c = createConstraints("gridbag", [1 1 2 5], [1 1], "both", "right");
main_frame = uicontrol(...
    "style", "frame",...
    "constraints", c,...
    "layout", "gridbag",...
    "tag", "main_frame");

// Text1
c = createConstraints("gridbag", [1 1 2 1], [1 1/5], "both", "upper");
text = uicontrol(...
    "parent", main_frame,...
    "style", "text",...
    "string", " Load a music and an impulse reponse... ",...
    "horizontalAlignment", "center",...
    "constraints", c,...
    "tag", "text1");

// Load music button
c = createConstraints("gridbag", [1 2 1 1], [1/2 1/5], "both", "upper");
loadButton = uicontrol(...
    "parent", main_frame,...
    "style", "pushbutton",...
    "icon", "document-open",...
    "string", " Load a sound ",...
    "constraints", c,...
    "callback", "[si, rate] = loadFile()",...
    "tag", "loadButton");

// Load impulse response button
c = createConstraints("gridbag", [2 2 1 1], [1/2 1/5], "both", "upper");
loadButton = uicontrol(...
    "parent", main_frame,...
    "style", "pushbutton",...
    "icon", "document-open",...
    "string", " Load an IR ",...
    "constraints", c,...
    "callback", "ir = loadFile()",...
    "tag", "loadButton");

// Text2
c = createConstraints("gridbag", [1 3 2 1], [1 1/5], "both", "upper");
text = uicontrol(...
    "parent", main_frame,...
    "style", "text",...
    "string", " ...and play the convolved output ",...
    "horizontalAlignment", "center",...
    "constraints", c,...
    "tag", "text2");

// Play original button
c = createConstraints("gridbag", [1 4 1 1], [1/2 1/5], "both", "upper");
playButton = uicontrol(...
    "parent", main_frame,...
    "style", "pushbutton",...
    "string", "Play the original sound",...
    "constraints", c,...
    "callback", "play(si, rate)",...
    "tag", "play");

// Play tenth button
c = createConstraints("gridbag", [1 5 1 1], [1/2 1/5], "both", "upper");
playButton = uicontrol(...
    "parent", main_frame,...
    "style", "pushbutton",...
    "string", "Play a tenth of the sound (if too long)",...
    "constraints", c,...
    "callback", "playTenth(si, rate)",...
    "tag", "play");

// Play convolved button
c = createConstraints("gridbag", [2 4 1 1], [1/2 1/5], "both", "upper");
playButton = uicontrol(...
    "parent", main_frame,...
    "style", "pushbutton",...
    "string", "Play the convolved sound",...
    "constraints", c,...
    "callback", "playConvol(si, rate, ir)",...
    "tag", "play");

// Play tenth of the convoled button
c = createConstraints("gridbag", [2 5 1 1], [1/2 1/5], "both", "upper");
playButton = uicontrol(...
    "parent", main_frame,...
    "style", "pushbutton",...
    "string", "Play a tenth of the convolved sound",...
    "constraints", c,...
    "callback", "playConvolTenth(si, rate, ir)",...
    "tag", "play");

function [si, rate] = loadFile()
    [filename, path] = uigetfile("*.*", pwd(), "Select the sound file (.wav)");

    if filename <> "" then
        if isfile(path+filesep()+filename) then
            [path, fname, extension] = fileparts(path+filesep()+filename);
            if extension <> ".wav" then
                messagebox("Wrong extension: WAV file expected", "Wrong extension", "error");
                updateStatusBar(" Erroneous file");
            else

                // File exists and is a .wav one
                [si, rate] = wavread(path+filesep()+filename);
                if ismatrix(si)
                    // Stereo sound, only keep an average of the two signals
                    if size(si, "r") == 2 then
                        si = (si(1, :) + si(2, :))/2;
                    else
                        si = (si(:, 1) + si(:, 2))/2;
                    end
                    updateStatusBar(" File loaded: "+fname+extension+"   Converted to mono");
                else
                    updateStatusBar(" File loaded: "+fname+extension);
                end

            end
        else
            messagebox("Please enter an existing file name", "Wrong file name", "error");
            updateStatusBar(" Erroneous file");
        end
    end
endfunction

function play(si, rate)
    updateStatusBar(" Playing the original sound...");
    playsnd(si, rate);
    updateStatusBar("");
endfunction

function playTenth(si, rate)
    pieceTenth = si(:, 1:floor(size(si,"c")/10));
    updateStatusBar(" Playing a tenth of the original sound...");
    playsnd(pieceTenth, rate);
    updateStatusBar("");
endfunction

function playConvol(si, rate, ir)
    updateStatusBar(" Computing and playing the convolved sound...");
    so = convol(ir, si);
    playsnd(so, rate);
    updateStatusBar("");
endfunction

function playConvolTenth(si, rate, ir)
    updateStatusBar(" Computing and playing a tenth of the convolved sound...");
    siTenth = si(:, 1:floor(size(si,"c")/10));
    soTenth = convol(ir, siTenth);
    playsnd(soTenth, rate);
    updateStatusBar("");
endfunction

function updateStatusBar(msg)
    set("main_figure", "info_message", msg);
endfunction

funcprot(p);

// Display the GUI
fig.visible = "on";
